%%
%             DO NOT USE WITH SYMMETRIC POINT SETS.
%
% This function recieves two point sets, {p_left}_i, {p_right}_i, and 
% computes the fiducial registration error (FRE) as defined in 
% Maurer et al. 1997:
% FRE = sqrt(sum(||T*{p_left}_i - perm({p_right}_i)||^2)/n)
%
% where T is a rigid transformation and "perm" is the permutation that 
% matches corresponding points between the two sets.
%
% FRE is a useful indicator of point set rigidity, which also reflects 
% localization precision. It does not reflect accuracy as any bias in 
% localization will be compensated for by the estimated transformation.
%
% The identification of corresponding points between the two point sets is
% based on the distances between points in each set, as a consequence:
%            DO NOT USE WITH SYMMETRIC POINT SETS.
%
% FRE definition:
% "Registration of Head Volume Images Using Implantable Fiducial Markers",
%  C. R. Maurer Jr., J. M. Fitzpatrick, M. Y. Wang, R. L. Galloway Jr., 
%  R. J. Maciunas, G. S. Allen, IEEE Trans. Med. Imag., vol. 16(4),
%  pp. 447-462, 1997.
%
% @param pointsInLeft A 3xn matrix where the columns are the points in the
%                     first (left) coordinate system.
%
% @param pointsInRight A 3xn matrix where the columns are the points in the
%                      second (right) coordinate system.
%
% @return FRE The fiducial registration error for the given point sets.
%
% @return distances The distances between corrosponding fiducials after 
%                   registration, order matches that of the "left" point 
%                   set.
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function [FRE, distances] = FREAnalysis(pointsInLeft, pointsInRight)

n = size(pointsInLeft,2);
                %check that both point sets are of the same cardinality
if size(pointsInRight,2) ~= n
  error('Point sets do not have the same cardinality.\n');
end
           %match the points from both point sets, this may fail
matchingPointsInRight = matchPointsBasedOnDistances(pointsInLeft, pointsInRight);
           %compute the transformation using the paired points
T = absoluteOrientation(pointsInLeft, matchingPointsInRight);

          %compute the FRE
transformedHomogenousPoints = T*[pointsInLeft;ones(1,n)];
squaredErrors =  (transformedHomogenousPoints(1:3,:) - matchingPointsInRight).^2;
FRE = sqrt(sum(squaredErrors(:))/n);
distances = sqrt(sum(squaredErrors,1));  
